C    *******************************************************************
        program md
	implicit none
	include 'md.inc'
C    *******************************************************************

C    *******************************************************************

        print *, '***************************************************'
        print *, 'Dinamica Molecular de atoms de Lennard Jones'
        print *, '***************************************************'
        print *, '* Calculo de la tension interfacial, liquido/aire *'
        print *, '* y construccion de un diagrama de fases          *'
        print *, '* para un sistema de Lennard Jones                *'
        print *, '* Tutorial realizado con el apoyo del proyecto    *'
        print *, '* PAPIME-DGAPA-UNAM-PE107419                      *'
        print *, '***************************************************'

        call config
        call start
	call run
	call finish
	
	stop
	end
	
	
C    *******************************************************************
        subroutine start
	implicit none
	include 'md.inc'
C    *******************************************************************
C    ** Local variables **
        integer         i
	doubleprecision dens
C    *******************************************************************

c	print *, '***************************************************'
c        print *, '* Calculo de la tension interfacial, liquido/aire *'
c        print *, '* y construccion de un diagrama de fases          *'
c        print *, '* para un sistema de Lennard Jones                *'
c        print *, '* Tutorial realizado con el apoyo del proyecto    *'
c        print *, '* PAPIME-DGAPA-PE107419                           *' 
c        print *, '***************************************************'
	
        print *, 'Dar el archivo para guardar la ultima configuracion'
        read  (*,'(a)') rundat
        write (*,'('' file     '',a)') rundat
        print *,'Archivo para guardar los datos la tension'
        read  (*,'(a)') disnam
        write (*,'('' file      '',a)') disnam
	print *, 'Numero de bloques'
	read  *, nblock
	print *, 'Numero de bloques = ', nblock
	print *, 'Numero de ciclos por bloque'
	read  *, nstep
	print *, 'Numero de ciclos  = ', nstep
	print *, 'timestep'
	read  *, dt
	print *, 'Timestep         = ', dt
	print *, 'Opcion de emperatura'
	read  *, fix_temp
	print *, 'Opcion de emperatura = ', fix_temp
	print *, 'Temperatura requerida'
	read  *, req_temp
	print *, 'Temperatura requerida = ', req_temp
	print *, 'Distancia de corte para el potencial'
	read  *, rcut
	
	dens = dble(n)/(boxx*boxy*boxz)
        detz = boxz/500
        nrad = int(anint(boxz/detz))

	call force
	call kinetic
	
	eng   = kin + pot
	temp  = 2.0d0*kin/3.0d0
c 	press = dens * ( temp + vir )
	
	print *, 'Valores Iniciales'
	print *, 'Energia     = ', eng
	print *, 'Cinetica    = ', kin
	print *, 'Potencial   = ', pot
	print *, 'Temperatura = ', temp
c	print *, 'Presion     = ', press
	
	return
	end
	
	
C    *******************************************************************
        subroutine run
	implicit none
	include 'md.inc'
C    *******************************************************************
C    ** Local variables **
        integer         j, block, step
	doubleprecision dens,asqpot, asqprxy,flp, flpx,area,stt
	doubleprecision fszz(500)
C    *******************************************************************

C    ** Zero run accumulators **
        runeng = 0.0d0
        runkin = 0.0d0
        runpot = 0.0d0
c        runvir = 0.0d0
        asqpot = 0.0d0
c        asqprxy= 0.0d0
        do j=1, nrad
         runrad(j) = 0.0d0
        enddo
        do j=1,nrad
         fsxy(j) = 0.0d0
         fsz(j)  = 0.0d0
        enddo
        area = boxx*boxy

        write(*,'(1x,''Bloque'',''    Energia'',''   cinetica'',
     :            '' Potencial'',''      Temp'',''     Pres'')')
     	
        do block = 1, nblock

C       ** Zero run accumulators **
           blkeng = 0.0d0
           blkkin = 0.0d0
           blkpot = 0.0d0
c           blkvir = 0.0d0

	   do step = 1, nstep
	      call movvel
	      call movpos
	      call force
              eng = kin + pot
	      runeng = runeng + eng
	      runkin = runkin + kin
	      runpot = runpot + pot
c	      runvir = runvir + vir
	      blkeng = blkeng + eng
	      blkkin = blkkin + kin
	      blkpot = blkpot + pot
c	      blkvir = blkvir + vir
              asqpot = asqpot + pot*pot

	      call locden
              do j=1, nrad
               runrad(j) = runrad(j)+gdp(j)
              enddo 
	   enddo


C       ** Normalize block accumulators **
           blkeng = blkeng / dble(nstep)
           blkkin = blkkin / dble(nstep)
           blkpot = blkpot / dble(nstep)
c           blkvir = blkvir / dble(nstep) 

	   dens   = dble(n)/(boxx*boxy*boxz)
	   temp   = 2.0d0*blkkin/3.0d0
c	   press  = dens * ( temp + blkvir )
c           asqprxy= asqprxy + press*press

	   write(*,'(1x,i5,5f10.4)') 
     :          block, blkeng, blkkin, blkpot, temp
	enddo
c        print *, fsxy(100),  fsz(100)
        do j=1, nrad
         runrad(j) = (runrad(j)/dble(nstep*nblock))/(area*detz)
        enddo 
        do j=1,nrad
         fsxy(j) = fsxy(j)/(area*dble(nstep*nblock))
         fsz(j)  = fsz(j)/(area*dble(nstep*nblock))
         fszz(j) = fsz(j) - fsxy(j)
        enddo
        stt=0.0
        do j=1,nrad
          stt= stt+ fszz(j)*detz
       enddo
c       print *, fsxy(100),  fsz(100),  runrad(100)
        do j=1,nrad
         fsxy(j)= runrad(j)*temp + fsxy(j)
         fsz(j) = runrad(j)*temp + fsz(j)
        enddo

c        print *, fsxy(100),  fsz(100)
	
C    ** Normalize run accumulators **
        runeng = runeng / dble(nstep*nblock)
        runkin = runkin / dble(nstep*nblock)
        runpot = runpot / dble(nstep*nblock)
c        runvir = runvir / dble(nstep*nblock)
        asqpot = (asqpot/dble(nstep*nblock))  - runpot**2

	dens   = dble(n)/(boxx*boxy*boxz)
	temp   = 2.0d0*runkin/3.0d0
c	press  = dens * ( temp + runvir )
	asqprxy= (asqprxy/nblock) - press**2
        if (asqpot .gt. 0.0d0)  flp  = sqrt(asqpot)
c        if (asqprxy .gt. 0.0d0) flpx = sqrt(asqprxy)


	write(*,'(1x,55(''-''))')
	write(*,'(1x,5x,5f10.4)') 
     :       runeng, runkin, runpot, temp
	write(*,'(1x,55(''-''))')
        write(*,'(1x,'' ** fluctation values ** '')')
        write(*,'(1x,5x,8f10.4)') flp

	
	return
	end


C    *******************************************************************
        subroutine finish
	implicit none
	include 'md.inc'
C    *******************************************************************
C    ** Local variables **
	integer         i
	doubleprecision dens
C    *******************************************************************
	
        dens = dble(n)/(boxx*boxy*boxz)
        eng   = kin + pot
        temp = 2.0d0*kin/3.0d0
c        press = dens * ( temp + vir )
         
	print *, 'Valores Finales'
	print *, 'Energia     = ', eng
	print *, 'cinetica    = ', kin
	print *, 'Potencial   = ', pot
	print *, 'Temperatura = ', temp
c	print *, 'Presion     = ', press

c         do i=1, n
c          rz(i) = rz(i) + boxx/2.0d0
c         enddo
	
	open(unit=3,file=rundat,status='unknown')
	write(3,'(1x,i5)') n
	write(3,'(1x,3f10.5)') boxx, boxy, boxz
        do i = 1, n
	   write(3,'(1x,6f10.5)') rx(i),ry(i),rz(i),vx(i),vy(i),vz(i)
	enddo
	close(unit=3)

        open(unit=3,file=disnam,status='unknown')
C        write(3,'(1x,''   k'',''   r'',''   g(2)'')')
        do i=1, nrad
          write(3,'(1x,i5,5f10.5)') i,i*detz-detz/2d0,runrad(i),fsz(i),
     :      fsxy(i),fsz(i)-fsxy(i)
        enddo
        close(unit=3) 


        return
	end


C    *******************************************************************
        subroutine movvel
	implicit none
	include 'md.inc'
C    *******************************************************************
C    ** Local variables **
	integer         i
	doubleprecision dt2, factor
C    *******************************************************************
	
	dt2 = dt / 2.0d0
	kin = 0.0d0

C    ** First half of velocity move **
C    ** Calculate kinetic energy    **
	do i = 1, n
	   vx(i) = vx(i) + dt2*fx(i)
	   vy(i) = vy(i) + dt2*fy(i)
	   vz(i) = vz(i) + dt2*fz(i)
	   kin = kin + vx(i)**2 + vy(i)**2 + vz(i)**2
	enddo
	kin = kin / 2.0d0
	kin = kin / dble(n)
	temp = 2.0d0 * kin / 3.0d0

C    ** Scale velocities if required **	
	if ( fix_temp ) then
	   factor = sqrt(req_temp/temp)
	   kin = 0.0d0
	   do i = 1, n
	      vx(i) = vx(i) * factor
	      vy(i) = vy(i) * factor
	      vz(i) = vz(i) * factor
	      kin = kin + vx(i)**2 + vy(i)**2 + vz(i)**2
	   enddo
	   kin = kin / 2.0d0
	   kin = kin / dble(n)
	   temp = 2.0d0 * kin / 3.0d0
	endif

C    ** Second half of velocity move **	
	do i = 1, n
	   vx(i) = vx(i) + dt2*fx(i)
	   vy(i) = vy(i) + dt2*fy(i)
	   vz(i) = vz(i) + dt2*fz(i)
	enddo

        return
	end

	
C    *******************************************************************
        subroutine movpos
	implicit none
	include 'md.inc'
C    *******************************************************************
C    ** Local variables **
	integer i
C    *******************************************************************

        do i = 1, n
	   rx(i) = rx(i) + dt*vx(i)
	   ry(i) = ry(i) + dt*vy(i)
	   rz(i) = rz(i) + dt*vz(i)
	   rx(i) = rx(i) - anint(rx(i)/boxx)*boxx
	   ry(i) = ry(i) - anint(ry(i)/boxy)*boxy
           rz(i) = rz(i) - anint(rz(i)/boxz)*boxz
	enddo
	
	return
	end


C    *******************************************************************
        subroutine force
	implicit none
	include 'md.inc'
C    *******************************************************************
C    ** Local variables **
	integer         i, j, w, wd, wx,wz
	doubleprecision rxi, ryi, rzi, fxi, fyi, fzi,fxyij,fzzij,zij
	doubleprecision rxij, ryij, rzij, rijsq, rcutsq, vcut
	doubleprecision r2ij, r6ij, r12ij, fij, vij,dife
	doubleprecision fxij, fyij, fzij
C    *******************************************************************

        rcutsq = rcut*rcut
	r2ij   = 1.0d0/rcutsq
	r6ij   = r2ij*r2ij*r2ij
	r12ij  = r6ij*r6ij
	vcut   = r12ij - r6ij
        dife = boxz/2.0
	
C    ** Zero energies and forces **
        pot = 0.0d0
c	vir = 0.0d0
	do i = 1, n
	   fx(i) = 0.0d0
	   fy(i) = 0.0d0
	   fz(i) = 0.0d0
	enddo

C    ** Outer loop over i **
        do i = 1, n-1
	   rxi = rx(i)
	   ryi = ry(i)
	   rzi = rz(i)
	   fxi = fx(i)
	   fyi = fy(i)
	   fzi = fz(i)
	   w   = int((rz(i)+dife)/detz)+1
C       ** Inner loop over j **
           do j = i+1, n
	      rxij = rxi - rx(j)
	      ryij = ryi - ry(j)
	      rzij = rzi - rz(j)
              wd   = int((rz(j)+dife)/detz)+1
              wz = abs(w-wd)+1
              if (abs(w-wd) .eq. 0) wz = 1               
c              rzij= wz*detz            
	      rxij = rxij - anint(rxij/boxx)*boxx
	      ryij = ryij - anint(ryij/boxy)*boxy
              rzij = rzij - anint(rzij/boxz)*boxz
              rijsq= rxij*rxij + ryij*ryij + rzij*rzij
	      if ( rijsq .lt. rcutsq ) then
	       r2ij  = 1.0d0/rijsq
	       r6ij  = r2ij*r2ij*r2ij
	       r12ij = r6ij*r6ij
	       vij   = r12ij - r6ij
	       fij   = (vij + r12ij)*r2ij
	       vij   = vij - vcut
	       pot   = pot + vij
	       fxij  = fij*rxij
	       fyij  = fij*ryij
	       fzij  = fij*rzij
c	       vir   = vir + rxij*fxij + ryij*fyij + rzij*fzij
	       fxi   = fxi + fxij
	       fyi   = fyi + fyij
	       fzi   = fzi + fzij
	       fx(j) = fx(j) - fxij
	       fy(j) = fy(j) - fyij
	       fz(j) = fz(j) - fzij 
               fxyij = (rxij*fxij + ryij*fyij)/(wz*detz)
               fzzij = rzij*fzij/(wz*detz)
c               fxyij = (rxij*fxij + ryij*fyij)/abs(rzij)
c               fzzij = rzij*fzij/abs(rzij)  
c               if (wz .eq. 1) fxyij = 0.0
c               if (wz .eq. 1) fzzij = 0.0
               if ((w-wd) .gt. 0) then 
                do wx=wd,w
                 fsxy(wx) = fsxy(wx) + fxyij*24.0/2.0
                 fsz(wx)  = fsz(wx) + fzzij*24.0/1.0
                enddo
               elseif ((w-wd) .le. 0) then
                do wx=w,wd
                 fsxy(wx) = fsxy(wx) + fxyij*24.0/2.0
                 fsz(wx)  = fsz(wx) + fzzij*24.0/1.0
                enddo
               endif
	      endif	 
	   enddo
           fx(i) = fxi
	   fy(i) = fyi
           fz(i) = fzi
	enddo
c        print *, fsxy(1),  fsz(1)
C    ** Incorporate correct numerical factors **
        pot = pot * 4.0d0
c	vir = vir * 24.0d0/3.0d0
        do i = 1, n
	   fx(i) = fx(i)*24.0d0
	   fy(i) = fy(i)*24.0d0
	   fz(i) = fz(i)*24.0d0
	enddo

        pot = pot/dble(n)
c	vir = vir/dble(n)


        return
	end




	
C    *******************************************************************
        subroutine kinetic
	implicit none
	include 'md.inc'
C    *******************************************************************
C    ** Local variables **
	integer         i
	doubleprecision dt2, vxi, vyi, vzi
C    *******************************************************************
	
	dt2 = dt / 2.0d0
	kin = 0.0d0

C    ** First half of velocity move **
C    ** Calculate kinetic energy    **
	do i = 1, n
	   vxi = vx(i) + dt2*fx(i)
	   vyi = vy(i) + dt2*fy(i)
	   vzi = vz(i) + dt2*fz(i)
	   kin = kin + vxi**2 + vyi**2 + vzi**2
	enddo
	kin = kin / 2.0d0
	kin = kin / dble(n)
	return
	end

C    *******************************************************************
        subroutine  locden
        implicit none
        include 'md.inc'
C    *******************************************************************
        integer         i, w
C    *******************************************************************

        detz = boxz/nrad
        do i=1, nrad
          gdp(i)  = 0.0
        enddo
        do i=1, n
          w  = int((rz(i)+boxz/2.0)/detz)+1
          if (w .le. nrad) then
            gdp(w) = gdp(w)+1
          endif
        enddo
                  
        return
        end 


C    *******************************************************************
        subroutine  config
	implicit none
	include 'md.inc'
C    *******************************************************************
C    ** Local variables **
        integer         i
        character       name*20
C    *******************************************************************

        print *,'Dar archivo inicial'
        read  (*,'(a)') name
        write(*,'('' file       '',a)') name

C    *******      read initial configuration      *******
        open (unit=3, file= name, status= 'old')
         read(3,*) n
         read(3,*)  boxx, boxy, boxz
          do i=1, n
c             read(3,*) rx(i),ry(i),rz(i)
             read(3,*) rx(i),ry(i),rz(i),vx(i),vy(i),vz(i)
          enddo
        close (unit=3)

c         do i=1, n
c             rz(i) = rz(i) - boxz/2.0d0
c         enddo
    
         return 
         end

